@@ -1,24 +1,12 @@
 package Babble::Match;
 
-use Babble::Grammar;
-use Babble::SymbolGenerator;
+use PPR::X;
 use Mu;
-use List::Util 1.45;
+use re 'eval';
 
 ro 'top_rule';
 rwp 'text';
-
-lazy 'grammar' => sub {
-  $_[0]->can('parent')
-    ? $_[0]->parent->grammar
-    : Babble::Grammar->new
-  } => handles => [ 'grammar_regexp' ];
-
-lazy 'symbol_generator' => sub {
-  $_[0]->can('parent')
-    ? $_[0]->parent->symbol_generator
-    : Babble::SymbolGenerator->new
-  } => handles => [ 'gensym' ];
+lazy 'grammar_regexp' => sub { $PPR::X::GRAMMAR };
 
 lazy top_re => sub {
   my ($self) = @_;
@@ -26,7 +14,6 @@ lazy top_re => sub {
   return "\\A${top}\\Z";
 };
 
-my %SUBMATCHES_COMPILE_CACHE;
 lazy submatches => sub {
   my ($self) = @_;
   return {} unless ref(my $top = $self->top_rule);
@@ -41,14 +28,7 @@ lazy submatches => sub {
       : $_
   } @$top;
   return {} unless @subrules;
-  my $submatch_re = qq[ \\A${re}\\Z ${\$self->grammar_regexp} ];
-  my @values = $self->text =~ (
-    $SUBMATCHES_COMPILE_CACHE{$submatch_re} ||= do {
-      use re 'eval';
-      my $re = qr/$submatch_re/x;
-      no re 'eval';
-      $re;
-    });
+  my @values = $self->text =~ /\A${re}\Z ${\$self->grammar_regexp}/x;
   die "Match failed" unless @values;
   my %submatches;
   require Babble::SubMatch;
@@ -68,15 +48,6 @@ lazy submatches => sub {
   return \%submatches;
 };
 
-sub subtexts {
-  my ($self, @names) = @_;
-  unless (@names) {
-    my %s = %{$self->submatches};
-    return +{ map +( $_ => $s{$_}->text ), keys %s };
-  }
-  map +($_ ? $_->text : undef), @{$self->submatches}{@names};
-}
-
 sub _rule_to_re {
   my $re = $_[1];
   return "(?&Perl${re})" unless ref($re);
@@ -91,18 +62,16 @@ sub is_valid {
 sub match_positions_of {
   my ($self, $of) = @_;
   our @F;
-  my $wrapped = $self->grammar->clone->extend_rule(
-                  $of => sub { '('.$_[0].')'.'(?{ push @Babble::Match::F, [ pos() - length($^N), length($^N) ] })' }
-                )->grammar_regexp;
+  my $wrapped = qr{(?(DEFINE)
+    (?<Perl${of}>((?&PerlStd${of}))(?{ push @F, [ pos() - length($^N), length($^N) ] }))
+  ) ${\$self->grammar_regexp}}x;
   my @found = do {
     local @F;
     local $_ = $self->text;
     /${\$self->top_re} ${wrapped}/x;
     @F;
   };
-  return map { [ split ',', $_ ] }
-          List::Util::uniqstr
-          map { join ",", @$_ } @found;
+  return @found;
 }
 
 sub each_match_of {
@@ -113,7 +82,7 @@ sub each_match_of {
   while (my $f = shift @found) {
     my $match = substr($self->text, $f->[0], $f->[1]);
     my $obj = Babble::SubMatch->new(
-                top_rule => $of,
+                top_rule => $self->_rule_to_re($of),
                 start => $f->[0],
                 text => $match,
                 parent => $self,
@@ -135,11 +104,13 @@ sub each_match_of {
 sub each_match_within {
   my ($self, $within, $rule, $call) = @_;
   my $match_re = $self->_rule_to_re($rule);
-  my $extend_grammar = $self->grammar->clone;
-  $extend_grammar->add_rule(
-    BabbleInnerMatch => $match_re,
-  )->augment_rule($within => '(?&PerlBabbleInnerMatch)');
-  local $self->{grammar} = $extend_grammar;
+  my $extend_grammar = qq{
+    (?(DEFINE)
+      (?<PerlBabbleInnerMatch>(?<PerlStdBabbleInnerMatch> ${match_re}))
+      (?<Perl${within}> (?&PerlBabbleInnerMatch) | (?&PerlStd${within}))
+    )
+  };
+  local $self->{grammar_regexp} = join "\n", $extend_grammar, $self->grammar_regexp;
   $self->each_match_of(BabbleInnerMatch => sub {
     $_[0]->{top_rule} = $rule; # intentionally hacky, should go away (or rwp) later
     $call->($_[0]);
@@ -153,7 +124,6 @@ sub replace_substring {
   substr($text, $start, $length, $replace);
   $self->_set_text($text);
   foreach my $submatch (values %{$self->submatches}) {
-    next unless defined $submatch;
     if ($submatch->start > $start) {
       $submatch->{start} += length($replace) - $length;
     }
@@ -161,33 +131,4 @@ sub replace_substring {
   return $self;
 }
 
-sub remove_use_argument {
-  my ($self, $use, $argument, $keep_empty) = @_;
-  $self->each_match_within(
-    UseStatement =>
-    [ "use\\s+${use}\\s+", [ explist => '.*?' ], ';' ],
-    sub {
-      my ($m) = @_;
-      my $explist = $m->submatches->{explist};
-      return unless my @explist_names = eval $explist->text;
-      my @remain = grep $_ ne $argument, @explist_names;
-      return unless @remain < @explist_names;
-      unless (@remain) {
-        ($keep_empty ? $explist : $m)->replace_text('');
-        return;
-      }
-      $explist->replace_text('qw('.join(' ', @remain).')');
-    }
-  );
-}
-
-sub remove_use_statement {
-  my ($self, $use) = @_;
-  $self->each_match_within(
-    UseStatement =>
-    [ "use\\s+${use}.*?;" ],
-    sub { shift->replace_text('') },
-  );
-}
-
 1;
